package xhttp3.internal.connection;

import java.io.IOException;
import java.lang.ref.Reference;
import java.lang.ref.WeakReference;
import xhttp3.Address;
import xhttp3.ConnectionPool;
import xhttp3.OkHttpClient;
import xhttp3.Route;
import xhttp3.internal.Internal;
import xhttp3.internal.Util;
import xhttp3.internal.framed.ErrorCode;
import xhttp3.internal.framed.StreamResetException;
import xhttp3.internal.http.HttpStream;

public final class StreamAllocation {
   public final Address address;
   private Route route;
   private final ConnectionPool connectionPool;
   private final RouteSelector routeSelector;
   private int refusedStreamCount;
   private RealConnection connection;
   private boolean released;
   private boolean canceled;
   private HttpStream stream;

   public StreamAllocation(ConnectionPool connectionPool, Address address) {
      this.connectionPool = connectionPool;
      this.address = address;
      RouteSelector connectionPool1;
      connectionPool1 = new RouteSelector.<init>(address, this.routeDatabase());
      this.routeSelector = connectionPool1;
   }

   private RealConnection findHealthyConnection(int connectTimeout, int readTimeout, int writeTimeout, boolean connectionRetryEnabled, boolean doExtensiveHealthChecks) {
      while(true) {
         RealConnection var6 = this.findConnection(connectTimeout, readTimeout, writeTimeout, connectionRetryEnabled);
         ConnectionPool var7 = this.connectionPool;
         RealConnection var10000 = var6;
         synchronized(var7){}

         boolean var10001;
         Throwable var20;
         int var21;
         try {
            var21 = var10000.successCount;
         } catch (Throwable var19) {
            var20 = var19;
            var10001 = false;
            throw var20;
         }

         if (var21 == 0) {
            try {
               return var6;
            } catch (Throwable var17) {
               var20 = var17;
               var10001 = false;
               throw var20;
            }
         } else {
            try {
               var10000 = var6;
               var10001 = doExtensiveHealthChecks;
            } catch (Throwable var18) {
               var20 = var18;
               var10001 = false;
               throw var20;
            }

            if (var10000.isHealthy(var10001)) {
               return var6;
            }

            this.noNewStreams();
         }
      }
   }

   private RealConnection findConnection(int connectTimeout, int readTimeout, int writeTimeout, boolean connectionRetryEnabled) {
      StreamAllocation var10000 = this;
      synchronized(this.connectionPool){}

      Throwable var334;
      boolean var335;
      boolean var10001;
      try {
         var335 = var10000.released;
      } catch (Throwable var330) {
         var334 = var330;
         var10001 = false;
         throw var334;
      }

      if (!var335) {
         HttpStream var336;
         try {
            var336 = this.stream;
         } catch (Throwable var328) {
            var334 = var328;
            var10001 = false;
            throw var334;
         }

         if (var336 == null) {
            try {
               var335 = this.canceled;
            } catch (Throwable var326) {
               var334 = var326;
               var10001 = false;
               throw var334;
            }

            if (!var335) {
               RealConnection var338;
               try {
                  var338 = this.connection;
               } catch (Throwable var324) {
                  var334 = var324;
                  var10001 = false;
                  throw var334;
               }

               RealConnection var6 = var338;
               if (var338 != null) {
                  try {
                     var335 = var6.noNewStreams;
                  } catch (Throwable var323) {
                     var334 = var323;
                     var10001 = false;
                     throw var334;
                  }

                  if (!var335) {
                     try {
                        return var6;
                     } catch (Throwable var317) {
                        var334 = var317;
                        var10001 = false;
                        throw var334;
                     }
                  }
               }

               ConnectionPool var10002;
               StreamAllocation var340;
               Internal var342;
               try {
                  var342 = Internal.instance;
                  var340 = this;
                  var10002 = this.connectionPool;
               } catch (Throwable var322) {
                  var334 = var322;
                  var10001 = false;
                  throw var334;
               }

               ConnectionPool var332 = var10002;

               try {
                  var338 = var342.get(var332, var340.address, this);
               } catch (Throwable var321) {
                  var334 = var321;
                  var10001 = false;
                  throw var334;
               }

               var6 = var338;
               if (var338 != null) {
                  try {
                     this.connection = var6;
                     return var6;
                  } catch (Throwable var318) {
                     var334 = var318;
                     var10001 = false;
                     throw var334;
                  }
               } else {
                  Route var344;
                  try {
                     var344 = this.route;
                  } catch (Throwable var320) {
                     var334 = var320;
                     var10001 = false;
                     throw var334;
                  }

                  Route var333 = var344;

                  try {
                     ;
                  } catch (Throwable var319) {
                     var334 = var319;
                     var10001 = false;
                     throw var334;
                  }

                  if (var344 == null) {
                     var333 = this.routeSelector.next();
                     ConnectionPool var5;
                     ConnectionPool var345 = var5 = this.connectionPool;
                     var340 = this;
                     StreamAllocation var337 = this;
                     Route var10003 = var333;
                     synchronized(var5) {
                        var337.route = var10003;
                        var340.refusedStreamCount = 0;
                     }
                  }

                  var10000 = this;
                  var340 = this;
                  RealConnection var331;
                  RealConnection var10004 = var331 = new RealConnection;
                  var10004.<init>(var333);
                  this.acquire(var10004);
                  var332 = this.connectionPool;
                  RealConnection var339 = var331;
                  synchronized(var332){}

                  try {
                     Internal.instance.put(this.connectionPool, var331);
                     var340.connection = var339;
                     var335 = var10000.canceled;
                  } catch (Throwable var316) {
                     var334 = var316;
                     var10001 = false;
                     throw var334;
                  }

                  if (!var335) {
                     int var341;
                     StreamAllocation var10005;
                     int var343;
                     try {
                        var338 = var331;
                        var340 = this;
                        var339 = var331;
                        var341 = connectTimeout;
                        var343 = readTimeout;
                        var10005 = this;
                     } catch (Throwable var314) {
                        var334 = var314;
                        var10001 = false;
                        throw var334;
                     }

                     var339.connect(var341, var343, writeTimeout, var10005.address.connectionSpecs(), connectionRetryEnabled);
                     var340.routeDatabase().connected(var331.route());
                     return var338;
                  } else {
                     try {
                        throw new IOException("Canceled");
                     } catch (Throwable var315) {
                        var334 = var315;
                        var10001 = false;
                        throw var334;
                     }
                  }
               }
            } else {
               try {
                  throw new IOException("Canceled");
               } catch (Throwable var325) {
                  var334 = var325;
                  var10001 = false;
                  throw var334;
               }
            }
         } else {
            try {
               throw new IllegalStateException("stream != null");
            } catch (Throwable var327) {
               var334 = var327;
               var10001 = false;
               throw var334;
            }
         }
      } else {
         try {
            throw new IllegalStateException("released");
         } catch (Throwable var329) {
            var334 = var329;
            var10001 = false;
            throw var334;
         }
      }
   }

   private RouteDatabase routeDatabase() {
      return Internal.instance.routeDatabase(this.connectionPool);
   }

   private void deallocate(boolean noNewStreams, boolean released, boolean streamFinished) {
      RealConnection streamFinished1;
      RealConnection var219;
      label1598: {
         boolean var10001;
         Throwable var218;
         label1599: {
            boolean var10000 = streamFinished;
            streamFinished1 = null;
            ConnectionPool var4;
            synchronized(var4 = this.connectionPool){}
            if (var10000) {
               try {
                  this.stream = null;
               } catch (Throwable var214) {
                  var218 = var214;
                  var10001 = false;
                  break label1599;
               }
            }

            if (released) {
               try {
                  this.released = true;
               } catch (Throwable var213) {
                  var218 = var213;
                  var10001 = false;
                  break label1599;
               }
            }

            try {
               var219 = this.connection;
            } catch (Throwable var212) {
               var218 = var212;
               var10001 = false;
               break label1599;
            }

            RealConnection released1 = var219;
            if (var219 != null) {
               if (noNewStreams) {
                  try {
                     released1.noNewStreams = true;
                  } catch (Throwable var211) {
                     var218 = var211;
                     var10001 = false;
                     break label1599;
                  }
               }

               HttpStream var220;
               try {
                  var220 = this.stream;
               } catch (Throwable var210) {
                  var218 = var210;
                  var10001 = false;
                  break label1599;
               }

               if (var220 == null) {
                  label1601: {
                     try {
                        var10000 = this.released;
                     } catch (Throwable var209) {
                        var218 = var209;
                        var10001 = false;
                        break label1599;
                     }

                     if (!var10000) {
                        try {
                           var10000 = released1.noNewStreams;
                        } catch (Throwable var208) {
                           var218 = var208;
                           var10001 = false;
                           break label1599;
                        }

                        if (!var10000) {
                           break label1601;
                        }
                     }

                     try {
                        this.release(released1);
                        var10000 = this.connection.allocations.isEmpty();
                     } catch (Throwable var207) {
                        var218 = var207;
                        var10001 = false;
                        break label1599;
                     }

                     if (var10000) {
                        ConnectionPool var10002;
                        StreamAllocation var221;
                        Internal var222;
                        try {
                           this.connection.idleAtNanos = System.nanoTime();
                           var222 = Internal.instance;
                           var221 = this;
                           var10002 = this.connectionPool;
                        } catch (Throwable var206) {
                           var218 = var206;
                           var10001 = false;
                           break label1599;
                        }

                        ConnectionPool noNewStreams1 = var10002;

                        try {
                           var10000 = var222.connectionBecameIdle(noNewStreams1, var221.connection);
                        } catch (Throwable var205) {
                           var218 = var205;
                           var10001 = false;
                           break label1599;
                        }

                        if (var10000) {
                           try {
                              var219 = this.connection;
                           } catch (Throwable var204) {
                              var218 = var204;
                              var10001 = false;
                              break label1599;
                           }

                           streamFinished1 = var219;
                        }
                     }

                     try {
                        this.connection = null;
                     } catch (Throwable var203) {
                        var218 = var203;
                        var10001 = false;
                        break label1599;
                     }
                  }
               }
            }

            label1553:
            try {
               var219 = streamFinished1;
               break label1598;
            } catch (Throwable var202) {
               var218 = var202;
               var10001 = false;
               break label1553;
            }
         }

         while(true) {
            try {
               throw var218;
            } catch (Throwable var201) {
               var218 = var201;
               var10001 = false;
               continue;
            }
         }
      }

      if (var219 != null) {
         Util.closeQuietly(streamFinished1.socket());
      }

   }

   private void release(RealConnection connection) {
      int var2 = 0;

      for(int var3 = connection.allocations.size(); var2 < var3; ++var2) {
         if (((Reference)connection.allocations.get(var2)).get() == this) {
            connection.allocations.remove(var2);
            return;
         }
      }

      throw new IllegalStateException();
   }

   public HttpStream newStream(OkHttpClient param1, boolean param2) {
      // $FF: Couldn't be decompiled
   }

   public void streamFinished(boolean noNewStreams, HttpStream stream) {
      ConnectionPool var3;
      synchronized(var3 = this.connectionPool){}
      Throwable var10000;
      boolean var10001;
      if (stream != null) {
         HttpStream var34;
         HttpStream var35;
         try {
            var34 = stream;
            var35 = this.stream;
         } catch (Throwable var33) {
            var10000 = var33;
            var10001 = false;
            throw var10000;
         }

         if (var34 == var35) {
            if (!noNewStreams) {
               RealConnection var36;
               int var37;
               try {
                  var36 = this.connection;
                  var37 = var36.successCount;
               } catch (Throwable var31) {
                  var10000 = var31;
                  var10001 = false;
                  throw var10000;
               }

               ++var37;

               try {
                  var36.successCount = var37;
               } catch (Throwable var30) {
                  var10000 = var30;
                  var10001 = false;
                  throw var10000;
               }
            }

            StreamAllocation var38;
            try {
               var38 = this;
               var10001 = noNewStreams;
            } catch (Throwable var29) {
               var10000 = var29;
               var10001 = false;
               throw var10000;
            }

            var38.deallocate(var10001, false, true);
            return;
         }
      }

      try {
         throw new IllegalStateException("expected " + this.stream + " but was " + stream);
      } catch (Throwable var32) {
         var10000 = var32;
         var10001 = false;
         throw var10000;
      }
   }

   public HttpStream stream() {
      StreamAllocation var10000 = this;
      synchronized(this.connectionPool) {
         return var10000.stream;
      }
   }

   public synchronized RealConnection connection() {
      return this.connection;
   }

   public void release() {
      this.deallocate(false, true, false);
   }

   public void noNewStreams() {
      this.deallocate(true, false, false);
   }

   public void cancel() {
      StreamAllocation var10000 = this;
      StreamAllocation var10001 = this;
      ConnectionPool var1;
      synchronized(var1 = this.connectionPool){}

      Throwable var16;
      HttpStream var17;
      boolean var18;
      try {
         var10001.canceled = true;
         var17 = var10000.stream;
      } catch (Throwable var14) {
         var16 = var14;
         var18 = false;
         throw var16;
      }

      HttpStream var2 = var17;

      RealConnection var10002;
      ConnectionPool var19;
      try {
         var19 = var1;
         var10002 = this.connection;
      } catch (Throwable var13) {
         var16 = var13;
         var18 = false;
         throw var16;
      }

      RealConnection var15 = var10002;

      try {
         ;
      } catch (Throwable var12) {
         var16 = var12;
         var18 = false;
         throw var16;
      }

      if (var17 != null) {
         var2.cancel();
      } else if (var15 != null) {
         var15.cancel();
      }

   }

   public void streamFailed(IOException e) {
      IOException var10000 = e;
      boolean var2 = false;
      synchronized(this.connectionPool){}

      boolean var10001;
      Throwable var218;
      boolean var219;
      try {
         var219 = var10000 instanceof StreamResetException;
      } catch (Throwable var214) {
         var218 = var214;
         var10001 = false;
         throw var218;
      }

      StreamAllocation var222;
      int var224;
      if (var219) {
         label1525: {
            ErrorCode var220;
            try {
               var220 = ((StreamResetException)e).errorCode;
            } catch (Throwable var213) {
               var218 = var213;
               var10001 = false;
               throw var218;
            }

            ErrorCode e1 = var220;

            ErrorCode var221;
            try {
               var221 = ErrorCode.REFUSED_STREAM;
            } catch (Throwable var212) {
               var218 = var212;
               var10001 = false;
               throw var218;
            }

            ErrorCode var4 = var221;
            if (var220 == var221) {
               int var223;
               try {
                  var222 = this;
                  var223 = this.refusedStreamCount;
               } catch (Throwable var211) {
                  var218 = var211;
                  var10001 = false;
                  throw var218;
               }

               ++var223;

               try {
                  var222.refusedStreamCount = var223;
               } catch (Throwable var210) {
                  var218 = var210;
                  var10001 = false;
                  throw var218;
               }
            }

            if (e1 == var4) {
               try {
                  var224 = this.refusedStreamCount;
               } catch (Throwable var209) {
                  var218 = var209;
                  var10001 = false;
                  throw var218;
               }

               if (var224 <= 1) {
                  break label1525;
               }
            }

            var222 = this;
            var2 = true;

            try {
               var222.route = null;
            } catch (Throwable var208) {
               var218 = var208;
               var10001 = false;
               throw var218;
            }
         }
      } else {
         RealConnection var225;
         try {
            var225 = this.connection;
         } catch (Throwable var207) {
            var218 = var207;
            var10001 = false;
            throw var218;
         }

         RealConnection var216 = var225;
         if (var225 != null) {
            try {
               var219 = var216.isMultiplexed();
            } catch (Throwable var206) {
               var218 = var206;
               var10001 = false;
               throw var218;
            }

            if (!var219) {
               var222 = this;
               var2 = true;

               try {
                  var224 = var222.connection.successCount;
               } catch (Throwable var205) {
                  var218 = var205;
                  var10001 = false;
                  throw var218;
               }

               if (var224 == 0) {
                  Route var226;
                  try {
                     var226 = this.route;
                  } catch (Throwable var204) {
                     var218 = var204;
                     var10001 = false;
                     throw var218;
                  }

                  Route var217 = var226;
                  if (var226 != null && e != null) {
                     try {
                        this.routeSelector.connectFailed(var217, e);
                     } catch (Throwable var203) {
                        var218 = var203;
                        var10001 = false;
                        throw var218;
                     }
                  }

                  try {
                     this.route = null;
                  } catch (Throwable var202) {
                     var218 = var202;
                     var10001 = false;
                     throw var218;
                  }
               }
            }
         }
      }

      try {
         var222 = this;
         var10001 = var2;
      } catch (Throwable var201) {
         var218 = var201;
         var10001 = false;
         throw var218;
      }

      var222.deallocate(var10001, false, true);
   }

   public void acquire(RealConnection connection) {
      connection.allocations.add(new WeakReference(this));
   }

   public boolean hasMoreRoutes() {
      return this.route != null || this.routeSelector.hasNext();
   }

   public String toString() {
      return this.address.toString();
   }
}
